<?php
/*
Plugin Name: GSP Assets Manager
Plugin URI: 
Description: GSP Assets Manager allow you to live review and control WordPress javascript files and stylesheets, Optimize google page speed rank within 5 minutes, no high technical experience required.
Author: Firas Abd Alrahman
Version: 1.8
Author URI: https://github.com/doonfrs
*/

require_once(__DIR__ . '/helpers.php');
require_once(__DIR__ . '/gsp_mgr.php');

$plugin = plugin_basename(__FILE__);
add_action('init', 'gsp_plugin_endpoint');
add_action('admin_init', 'gsp_plugin_endpoint');
add_filter("plugin_action_links_$plugin", 'gsp_plugin_settings_link');
add_action('wp_ajax_gsp_save', 'gsp_ajax_save');
add_action('wp_ajax_gsp_delete_role', 'gsp_ajax_delete_role');
add_action('wp_ajax_gsp_stop_manager', 'gsp_ajax_stop_manager');
add_action('wp_ajax_gsp_load_manager', 'gsp_ajax_load_manager');
add_action('wp_ajax_gsp_create_new_role', 'gsp_ajax_save_new_role');


if (isset($_POST['action']) && $_POST['action'] == 'gsp_save') {
    gsp_stop_wp_hide();
}

function gsp_install_plugin()
{
    gsp_plugin_endpoint();
}

register_activation_hook(__FILE__, 'gsp_install_plugin');


function gsp_uninstall_plugin()
{
}

register_deactivation_hook(__FILE__, 'gsp_uninstall_plugin');

function gsp_plugin_endpoint()
{
    if (current_user_can('administrator')) {
        add_action('admin_bar_menu', function ($admin_bar) {
            $admin_bar->add_menu(array(
                'id'    => 'gspmgr-item',
                'title' => 'GSP Assets Manager!',
                'href'  => is_admin() ?  site_url('?gspmgr-mode=1') : '?gspmgr-mode=1',
            ));
        }, 100);
    }
}

function gsp_plugin_settings_link($links)
{
    $settings_link = '<a href="' . site_url('?gspmgr-mode=1') . '">Run GSP Assets Manager!</a>';
    array_unshift($links, $settings_link);
    return $links;
}


function gsp_ajax_save()
{
    gsp_ajax_prepare();


    $nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : null;

    if (!wp_verify_nonce($nonce, 'gsp_save')) {
        die('Nonce value cannot be verified.');
    }

    if (!current_user_can('manage_options')) {
        die('You are not administrator');
    }


    GSPMgr::reset();

    $role = isset($_POST['role']) ? sanitize_text_field($_POST['role']) : null;
    if (!$role) {
        die('Role value missing');
    }


    $styles_async = isset($_POST['style_async']) ? (array) $_POST['style_async'] : [];
    $styles_async = array_map('esc_url_raw', $styles_async);

    foreach ($styles_async as $async_style) {
        GSPMgr::setAsset($async_style, GSPMgr::TYPE_CSS, $role, $disabled = null, $position = null, $async = true);
    }

    $styles_defer = isset($_POST['style_defer']) ? (array) $_POST['style_defer'] : [];
    $styles_defer = array_map('esc_url_raw', $styles_defer);

    foreach ($styles_defer as $defer_style) {
        GSPMgr::setAsset($defer_style, GSPMgr::TYPE_CSS, $role, $disabled = null, $position = null, $async = null, $defer = true);
    }

    $disabled_scripts = isset($_POST['disabled_scripts']) ? (array) $_POST['disabled_scripts'] : [];
    $disabled_scripts = array_map('esc_url_raw', $disabled_scripts);

    foreach ($disabled_scripts as $script) {
        GSPMgr::setAsset($script, GSPMgr::TYPE_JS, $role, $disabled = true);
    }

    $disabled_styles = isset($_POST['disabled_styles']) ? (array) $_POST['disabled_styles'] : [];
    $disabled_styles = array_map('esc_url_raw', $disabled_styles);

    foreach ($disabled_styles as $style) {
        GSPMgr::setAsset($style, GSPMgr::TYPE_CSS, $role, $disabled = true);
    }

    $scripts_pos = isset($_POST['script_pos']) ? (array) $_POST['script_pos'] : [];

    foreach ($scripts_pos as $pos) {
        $pos_pos = sanitize_text_field($pos['pos']);
        $pos_script = esc_url_raw($pos['script']);
        GSPMgr::setAsset($pos_script, GSPMgr::TYPE_JS, $role, $disabled = null, $position = $pos_pos);
    }


    $styles_pos = isset($_POST['style_pos']) ? (array) $_POST['style_pos'] : [];

    foreach ($styles_pos as $pos) {
        $pos_pos = sanitize_text_field($pos['pos']);
        $pos_style = esc_url_raw($pos['style']);
        GSPMgr::setAsset($pos_style, GSPMgr::TYPE_CSS, $role, $disabled = null, $position = $pos_pos);
    }

    $scripts_async = isset($_POST['script_async']) ? (array) $_POST['script_async'] : [];
    foreach ($scripts_async as $async_script) {
        $async_script = esc_url_raw($async_script);
        GSPMgr::setAsset($async_script, GSPMgr::TYPE_JS, $role, $disabled = null, $position = null, $async = true);
    }

    $scripts_defer = isset($_POST['script_defer']) ? (array) $_POST['script_defer'] : [];
    foreach ($scripts_defer as $defer_script) {
        $defer_script = esc_url_raw($defer_script);
        GSPMgr::setAsset($defer_script, GSPMgr::TYPE_JS, $role, $disabled = null, $position = null, $async = null, $defer = true);
    }


    GSPMgr::saveAssets($role);
    echo 'ok';
    die();
}



function gsp_ajax_save_new_role()
{
    gsp_ajax_prepare();

    $nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : null;


    if (!wp_verify_nonce($nonce, 'gsp_create_new_role')) {
        die('Nonce value cannot be verified.');
    }

    if (!current_user_can('manage_options')) {
        die('You are not administrator');
    }

    $role_name = isset($_POST['role']) ? sanitize_text_field($_POST['role']) : null;
    $role_key = isset($_POST['key']) ? sanitize_text_field($_POST['key']) : null;
    $urls = explode("\n", $_POST['urls']);
    for ($i = 0; $i < count($urls); $i++) {
        $urls[$i] = esc_url_raw(urldecode($urls[$i]));
    }

    //if role being edited, let's remove it first
    if ($role_key) {
        GSPMgr::deleteRole($role_key);
    }

    $role = GSPMgr::saveCustomRole($role_name, $urls);

    echo json_encode(['role' => $role->toArray(), 'success' => true]);
    die();
}


function gsp_ajax_load_manager()
{
    gsp_ajax_prepare();

    $nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : null;

    if (!wp_verify_nonce($nonce, 'gsp_load_manager')) {
        die('Nonce value cannot be verified.');
    }

    if (!current_user_can('manage_options')) {
        die('You are not administrator');
    }

    require_once(__DIR__ . '/manager.php');
    gsp_manager_render();

    die();
}



function gsp_ajax_stop_manager()
{
    gsp_ajax_prepare();

    gsp_start_session();
    unset($_SESSION['gspmgr-mode']);
    unset($_SESSION['gspmgr-role']);

    echo 'ok';
    die();
}



function gsp_ajax_delete_role()
{
    gsp_ajax_prepare();

    $role = isset($_POST['role']) ? sanitize_text_field($_POST['role']) : null;

    if (!$role) {
        die('Role not set');
    }

    $nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : null;

    if (!wp_verify_nonce($nonce, 'gsp_delete_role')) {
        die('Nonce value cannot be verified.');
    }

    if (!current_user_can('manage_options')) {
        die('You are not administrator');
    }

    GSPMgr::deleteRole($role);

    echo 'ok';
    die();
}

function gspeed_process_finaloutput($buffer)
{
    $softDelete = false;

    if (gsp_manager_mode()) {
        //keep stylesheets & scripts but disable them, this will help the tool to show/hide the asset
        // for live preview
        $softDelete = true;

        gsp_set_applied_role();
        gsp_set_matched_roles();
    }

    // Apply any filters to the final output
    return GSPMgr::filterOutput($buffer, gsp_getCurrentUrl(), $softDelete);
}

function gsp_buffer_start()
{
    ob_start('gspeed_process_finaloutput');
}

if (!gsp_should_ignore()) {
    add_action('init', function () {
        if (gsp_manager_mode()) {

            //prepare the manager and embed the required assets
            //the manager will be load the manager html as ajax
            //it is required to load it via ajax later to avoid 
            //any changes maybe done by theme js to the manager html structure
            require_once(__DIR__ . '/manager.php');
            gsp_manager_init();
        }
    });

    if (gsp_is_wp_hide()) {
        add_filter('wp-hide/ob_start_callback', 'gspeed_process_finaloutput');
    } else {
        add_action('wp_loaded', 'gsp_buffer_start', PHP_INT_MAX, 1);
    }
}
