<?php

function gsp_remove_param_from_url($url, $param)
{
    $base_url = strtok($url, '?');              // Get the base url
    $parsed_url = parse_url($url);              // Parse it 
    $query = isset($parsed_url['query']) ? $parsed_url['query'] : null;              // Get the query string
    parse_str($query, $parameters);           // Convert Parameters into array
    unset($parameters[$param]);               // Delete the one you want
    $new_query = http_build_query($parameters); // Rebuilt query string

    return $base_url . '?' . $new_query;            // Finally url is ready
}

function gsp_should_ignore()
{
    if (isset($_GET['gspmgr-ignore'])) return true;
    if (is_admin()) return true;
    return false;
}


function gsp_test_url_mode()
{
    if (isset($_GET['gspmgr-test-mode'])) return true;
    return false;
}

function gsp_is_wp_hide()
{
    global $wph;
    if (isset($wph) && is_object($wph) && get_class($wph) == 'WPH') {
        return true;
    }
    return false;
}

function gsp_stop_wp_hide()
{
    global $wph;
    if (isset($wph) && is_object($wph) && get_class($wph) == 'WPH') {
        //do not touch the output
        add_filter('wp-hide/ignore_ob_start_callback', function () {
            return true;
        });
    }
}

function gsp_ajax_prepare()
{
    if (gsp_is_wp_hide()) {
        gsp_stop_wp_hide();
        //refill post array because wp_hide modify it and I could not find a better way
        //by hooks because wphide run at very beginning
        parse_str(file_get_contents('php://input'), $_POST);
    }
}

function gsp_manager_mode()
{
    gsp_start_session();


    if (isset($_GET['gspmgr-ignore'])) return false;
    if (isset($_GET['customize_theme'])) return false;
    if (is_admin()) return false;
    if (is_404()) return false;

    //we don't init manager in ajax mode
    if (defined('DOING_AJAX') && DOING_AJAX) return false;
    if (!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') return false;
    if (wp_doing_ajax()) return false;
    if (strpos(gsp_getCurrentUrl(), site_url('/wp-json/')) !== false) return false;
    if (defined('DOING_CRON')) return false;

    //chrome in debug mode ask for .map file, we don't want to process this, it confuse the session
    if (substr(gsp_getCurrentUrl(), -4) == '.map') return false;

    if (isset($_GET['gspmgr-mode'])) return true;
    if (isset($_SESSION['gspmgr-mode'])) return true;

    return false;
}

function gsp_add_url_param($url, $param, $paramvalue)
{

    $url = gsp_cleanUrl($url);
    $url = gsp_remove_param_from_url($url, $param);
    $url = gsp_cleanUrl($url);
    if (parse_url($url, PHP_URL_QUERY)) {
        $url .= "&";
    } else {
        $url .= "?";
    }

    $url .= $param . '=' . $paramvalue;
    return $url;
}


function gsp_remove_gsp_url_params($url)
{
    $params = ['gspmgr-ignore', 'gspmgr-test-mode', 'gspmgr-role', 'gspmgr-mode'];

    foreach ($params as $param) {
        $url = gsp_remove_param_from_url($url, $param);
    }

    return $url;
}

function gsp_start_session()
{
    if (!session_id()) session_start();
}

function gsp_get_cur_url()
{
    return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
}

function gsp_cleanUrl($url)
{
    $verpatterns = [
        '/\?ver=.+/',
        '/&ver=.+/',
        '/;ver=.+/',
    ];

    foreach ($verpatterns as $pattern) {
        $url = preg_replace($pattern, '', $url);
    }
    if (($p = strrpos($url, '#')) !== false) {
        $url = substr($url, 0, $p);
    }

    for ($i = 0; $i < 5; $i++) {
        $url = rtrim($url, '&#;? /');
    }


    $url = htmlspecialchars_decode($url);

    return $url;
}

function gsp_getCurrentUrl()
{
    $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

    $url = str_replace('action=gspmgr', '', $actual_link);

    $url = trim($url, " \t\n\r\0\x0B?&");
    return $url;
}


function gsp_short_link($link)
{
    $ary = explode('/', $link);
    $ary = array_slice($ary, -2, 2, true);

    $text = implode('/', $ary);

    $text = substr($text, -50);
    return $text;
}

function gsp_get_wordpress_url_group($link)
{

    $group = ['type' => 'external', 'name' => null];

    $plugin_preg = '/' . rtrim(preg_quote(plugins_url(), '/'), '\\/') . '\/(.+?)\/' . '/';
    $theme_preg = '/' . rtrim(preg_quote(get_theme_root_uri(), '/'), '\\/') . '\/(.+?)\/' . '/';
    $include_preg = '/' . rtrim(preg_quote(includes_url(), '/'), '\\/') . '\/(.+?)\/' . '/';


    if (preg_match($plugin_preg, $link, $match) === 1) {
        $group['type'] = 'plugin';
        $group['name'] = $match[1];
    } elseif (preg_match($theme_preg, $link, $match) === 1) {
        $group['type'] = 'theme';
        $group['name'] = $match[1];
    } elseif (preg_match($include_preg, $link, $match) === 1) {
        $group['type'] = 'core';
        $group['name'] = $match[1];
    } else {
        $parse = parse_url($link);
        $group['name'] = $parse['host'];
    }

    return $group;
}
