//wait until document ready
document.addEventListener("DOMContentLoaded", gsp_init);

window.onerror = function (msg, url, lineNo, columnNo, error) {
  alert(
    "JavaScript Error Detected, please review browser console: " +
      msg +
      " - " +
      url +
      " - " +
      lineNo +
      " - " +
      columnNo
  );
  return false;
};

function gsp_init() {
  //load jquery if not loaded
  if (!window.jQuery) {
    // Load the script
    const script = document.createElement("script");
    script.id = "gspmgr-jquery";
    script.src = gspmgr.wp_jquery;
    script.type = "text/javascript";
    script.addEventListener("load", () => {
      jQuery.noConflict();
      gsp_load();
    });
    document.head.appendChild(script);
  } else {
    gsp_load();
  }
}

function gsp_ajax_fail(xhr, textStatus, errorThrown) {
  alert(
    "Error Requesting Data, please try again, Status: " +
      textStatus +
      ", Error Message: " +
      xhr.responseText +
      "\n For more details check browser console log"
  );
  console.log(xhr);
  console.log(xhr.statusText);
  console.log(textStatus);
  console.log(errorThrown);
}

function gsp_load() {
  jQuery
    .post(gspmgr.ajax_url, {
      action: "gsp_load_manager",
      gsp_url: document.location.href,
      nonce: gspmgr.gsp_load_manager_nonce,
    })
    .done((result) => {
      jQuery(document.body).append(result);
      gsp_run();
    })
    .fail(gsp_ajax_fail);
}

function gsp_run() {
  hideLoader();

  jQuery(document).on("click", "#gspmgr-matched-roles a", (e) => {
    e.preventDefault();
    jQuery("#gspmgr-roles")
      .val(jQuery(e.target).data("role"))
      .trigger("change");
  });

  jQuery(document).on("click", ".gspmgr-asset-element-group-toggle", (e) => {
    var group = jQuery(e.currentTarget).data("group");
    var checked = jQuery(e.currentTarget).is(":checked");
    jQuery(
      '.gspmgr-asset-element[data-group="' + group + '"] .gspmgr-toggle-asset'
    ).prop("checked", checked);
  });

  jQuery(document).on("change", "#gspmgr-roles", (e) => {
    showLoader("Switching Role...");
    var url = document.location.href;
    url = removeURLParam(url, "gspmgr-role");
    url = removeURLParam(url, "gspmgr-mode");
    url = cleanUrl(url);
    document.location =
      url + "?gspmgr-mode=1&gspmgr-role=" + jQuery(e.target).val();
  });

  jQuery(document).on("click", "#gspmgr-stop", (e) => {
    e.preventDefault();
    if (confirm("Are you sure do you want to stop GSP Assets Manager?")) {
      showLoader("Stopping...");
      jQuery
        .post(gspmgr.ajax_url, {
          action: "gsp_stop_manager",
          nonce: gspmgr.gsp_stop_manager_nonce,
        })
        .done((result) => {
          if (result != "ok") {
            alert("Error Stopping manager, please check your connection");
            return;
          }

          //because always will hide the loader
          setTimeout(() => {
            showLoader("Refreshing ...");
          }, 100);

          var url = document.location.href;
          url = removeURLParam(url, "gspmgr-role");
          url = removeURLParam(url, "gspmgr-mode");
          url = cleanUrl(url);
          document.location = url;
        })
        .fail(gsp_ajax_fail)
        .always(() => {
          hideLoader();
        });
    }
  });

  jQuery(document).on("click", "#gspmgr-add-role", (e) => {
    e.preventDefault();

    jQuery("#gspmgr-add-new-role-title").text("Create New Role");

    jQuery("#gsp-add-role-name").val("");
    jQuery("#gsp-add-role-urls").val("");
    jQuery("#gsp-add-role-key").val("");

    jQuery(".gspmgr-add-role-dialog").show();
    jQuery(".gspmgr-tool-content").hide();
  });

  jQuery(document).on("click", "#gspmgr-edit-role", (e) => {
    e.preventDefault();

    jQuery("#gsp-add-role-name").val(window.currentRoleObj.name);
    jQuery("#gsp-add-role-urls").val(
      window.currentRoleObj.urls ? window.currentRoleObj.urls.join("\n") : ""
    );
    jQuery("#gsp-add-role-key").val(window.currentRoleObj.key);
    jQuery("#gspmgr-add-new-role-title").text("Edit Role");
    jQuery(".gspmgr-add-role-dialog").show();
    jQuery(".gspmgr-tool-content").hide();
  });

  jQuery(document).on("click", "#gspmgr-new-role-cancel", (e) => {
    e.preventDefault();

    jQuery(".gspmgr-add-role-dialog").hide();
    jQuery(".gspmgr-tool-content").show();
  });

  jQuery(document).on("click", "#gspmgr-save-new-role", (e) => {
    e.preventDefault();

    showLoader("Creating Role...");
    var role = jQuery("#gsp-add-role-name").val();
    var urls = jQuery("#gsp-add-role-urls").val();
    var key = jQuery("#gsp-add-role-key").val();
    jQuery
      .post(gspmgr.ajax_url, {
        action: "gsp_create_new_role",
        nonce: gspmgr.gsp_create_new_role_nonce,
        role: role,
        urls: urls,
        key: key,
      })
      .done((result) => {
        result = jQuery.parseJSON(result);

        if (!result.success) {
          alert("Error Creating new role, please check your connection");
          hideLoader();
          return;
        }

        showLoader("Refreshing ...");

        var url = document.location.href;
        url = removeURLParam(url, "gspmgr-role");
        url = removeURLParam(url, "gspmgr-mode");
        url = cleanUrl(url);
        document.location =
          url + "?gspmgr-mode=1&gspmgr-role=" + result.role.key;
      })
      .fail(gsp_ajax_fail)
      .always(() => {
        hideLoader();
      });
  });

  jQuery(document).on("click", "#gspmgr-delete-role", (e) => {
    e.preventDefault();
    if (confirm("Are you sure do you want to delete the current role ?")) {
      showLoader("Deleting Role...");
      jQuery
        .post(gspmgr.ajax_url, {
          action: "gsp_delete_role",
          nonce: gspmgr.gsp_delete_role_nonce,
          role: jQuery("#gspmgr-roles").val(),
        })
        .done((result) => {
          if (result != "ok") {
            alert("Error Deleting role, please check your connection");
            hideLoader();
            return;
          }
          showLoader("Refreshing ...");

          var url = document.location.href;
          url = removeURLParam(url, "gspmgr-role");
          url = removeURLParam(url, "gspmgr-mode");
          url = cleanUrl(url);
          document.location = url + "?gspmgr-mode=1&gspmgr-role=all";
        })
        .fail(gsp_ajax_fail)
        .always(() => {
          hideLoader();
        });
    }
  });

  jQuery(document).on("click", ".gspmgr-header-button", (e) => {
    e.preventDefault();
    $target = jQuery(e.target).data("target");
    jQuery(".gspmgr-asset-list").hide();
    jQuery(".gspmgr-header-button").removeClass("gspmgr-active");
    jQuery($target).show();
    jQuery(e.target).addClass("gspmgr-active");
  });

  jQuery(document).on(
    "click",
    "#gspmgr-styles-list .gspmgr-asset-element input.gspmgr-toggle-style",
    (e) => {
      toggleStyle(
        jQuery(e.currentTarget).val(),
        jQuery(e.currentTarget).is(":checked")
      );
    }
  );

  jQuery(document).on("click", "#gspmgr-save", (e) => {
    e.preventDefault();
    showLoader("Saving ...");

    var disabledStyles = [];
    var disabledScripts = [];
    var stylePos = [];
    var scriptPos = [];
    var styleAsync = [];
    var scriptAsync = [];
    var styleDefer = [];
    var scriptDefer = [];    

    jQuery("#gspmgr-styles-list .gspmgr-asset-element")
      .find("input.gspmgr-toggle-style[type='checkbox']:not(:checked)")
      .each((index, e) => disabledStyles.push(jQuery(e).val()));

    jQuery("#gspmgr-scripts-list .gspmgr-asset-element")
      .find("input.gspmgr-toggle-script[type='checkbox']:not(:checked)")
      .each((index, e) => disabledScripts.push(jQuery(e).val()));

    jQuery("#gspmgr-scripts-list select[data-script-pos]").each((index, el) => {
      if (jQuery(el).val()) {
        scriptPos.push({
          script: jQuery(el).data("script-pos"),
          pos: jQuery(el).val(),
        });
      }
    });

    jQuery("#gspmgr-scripts-list input[data-script-async]:checked").each(
      (index, e) => {
        scriptAsync.push(jQuery(e).data("script-async"));
      }
    );

    jQuery("#gspmgr-scripts-list input[data-script-defer]:checked").each(
      (index, e) => {
        scriptDefer.push(jQuery(e).data("script-defer"));
      }
    );

    jQuery("#gspmgr-styles-list select[data-style-pos]").each((index, el) => {
      if (jQuery(el).val()) {
        stylePos.push({
          style: jQuery(el).data("style-pos"),
          pos: jQuery(el).val(),
        });
      }
    });

    jQuery("#gspmgr-styles-list input[data-style-async]:checked").each(
      (index, e) => {
        styleAsync.push(jQuery(e).data("style-async"));
      }
    );

    jQuery("#gspmgr-styles-list input[data-style-defer]:checked").each(
      (index, e) => {
        styleDefer.push(jQuery(e).data("style-defer"));
      }
    );    

    jQuery
      .post(gspmgr.ajax_url, {
        action: "gsp_save",
        nonce: gspmgr.gsp_save_nonce,
        role: jQuery("#gspmgr-roles").val(),
        disabled_scripts: disabledScripts,
        disabled_styles: disabledStyles,
        style_pos: stylePos,
        script_pos: scriptPos,
        script_async: scriptAsync,
        style_async: styleAsync,
        script_defer: scriptDefer,
        style_defer: styleDefer,
      })
      .done((result) => {
        if (result != "ok") {
          alert("Error Saving settings, please check your connection");
          hideLoader();
          return;
        }
        showLoader("Refreshing ...");
        location.reload();
      })
      .fail(gsp_ajax_fail)
      .always(() => {
        hideLoader();
      });
  });

  function removeURLParam(url, parameter) {
    //prefer to use l.search if you have a location/link object
    var urlparts = url.split("?");
    if (urlparts.length >= 2) {
      var prefix = encodeURIComponent(parameter) + "=";
      var pars = urlparts[1].split(/[&;]/g);

      //reverse iteration as may be destructive
      for (var i = pars.length; i-- > 0; ) {
        //idiom for string.startsWith
        if (pars[i].lastIndexOf(prefix, 0) !== -1) {
          pars.splice(i, 1);
        }
      }

      return urlparts[0] + (pars.length > 0 ? "?" + pars.join("&") : "");
    }
    return url;
  }

  function cleanUrl(url) {
    if (url.indexOf("#") > -1) {
      url = url.substr(0, url.indexOf("#"));
    }
    if (url.endsWith("&")) {
      url = url.substr(0, url.length - 1);
    }
    if (url.endsWith("?")) {
      url = url.substr(0, url.length - 1);
    }

    return url;
  }

  function setLoaderText(text) {
    jQuery("#gspmgr-loader .gspmgr-loader-text").text(text);
  }

  function hideLoader() {
    jQuery("#gspmgr-loader").fadeOut();
  }

  function showLoader(text) {
    setLoaderText(text);
    jQuery("#gspmgr-loader").fadeIn();
  }

  function toggleStyle(href, enable) {
    var links = document.getElementsByTagName("link");
    var found = false;
    for (l in links) {
      link = links[l];
      if (link.href && link.href.indexOf(href) > -1) {
        if (enable) {
          link.rel = "stylesheet";
        } else {
          link.rel = "removed";
        }
        found = true;
        break;
      }
    }
    if (!found && enable) {
      var link = document.createElement("link");
      link.href = href;
      link.rel = "stylesheet";
      frameDoc.head.append(link);
    }
  }
}
