<?php
function gsp_manager_init()
{

    if (!current_user_can('manage_options')) {
        wp_redirect(wp_login_url());
        die();
    }

    gsp_start_session();
    if (isset($_GET['gspmgr-role'])) {
        gsp_set_cur_role(sanitize_text_field($_GET['gspmgr-role']));
    }

    wp_enqueue_script('gspmgr', plugin_dir_url(__FILE__) . 'js/gspmgr.js');
    wp_enqueue_style('gspmgr', plugin_dir_url(__FILE__) . 'css/gspmgr.css');

    wp_localize_script(
        'gspmgr',
        'gspmgr',
        [
            'ajax_url' => admin_url('admin-ajax.php'),
            'gsp_save_nonce' => wp_create_nonce('gsp_save'),
            'gsp_delete_role_nonce' => wp_create_nonce('gsp_delete_role'),
            'gsp_stop_manager_nonce' => wp_create_nonce('gsp_stop_manager'),
            'gsp_load_manager_nonce' => wp_create_nonce('gsp_load_manager'),
            'gsp_create_new_role_nonce' => wp_create_nonce('gsp_create_new_role'),
            'wp_jquery' => includes_url('js/jquery/jquery.min.js'),
        ]
    );

    add_action('wp_footer', function () {
        echo '
    <div id="gspmgr-loader">
        <span class="gspmgr-loader-text">Loading GSP Assets Manager...</span>
    </div>
';
    });

    $_SESSION['gspmgr-mode'] = 1;


    wp_set_current_user(0);
}



function gsp_set_cur_role($role_key)
{
    $_SESSION['gspmgr-role'] = $role_key;
}

function gsp_get_cur_role()
{
    return isset($_SESSION['gspmgr-role']) ? sanitize_text_field($_SESSION['gspmgr-role']) : GSPMgr::ROLE_ALL;
}

/**
 * @return GspRole
 */
function gsp_get_applied_role()
{
    $key = gsp_get_applied_role_key();
    $role =  isset($_SESSION[$key]) ? $_SESSION[$key] : null;
    return $role;
}


function gsp_get_active_url()
{
    return isset($_POST['gsp_url']) ? esc_url_raw($_POST['gsp_url']) : null;
}

/**
 * @return GspRole
 */
function gsp_get_matched_roles()
{
    $key = gsp_get_matched_role_key();
    $role =  isset($_SESSION[$key]) ? $_SESSION[$key] : null;
    return $role;
}

function gsp_get_applied_role_key()
{
    return "gsp-applied-role-" . (gsp_get_active_url() ? gsp_get_active_url() : gsp_getCurrentUrl());
}

function gsp_get_matched_role_key()
{
    return "gsp-matched-role-" . (gsp_get_active_url() ? gsp_get_active_url() : gsp_getCurrentUrl());
}

//this function will be called while parsing the assets
//mangager will be able to fech this value
function gsp_set_applied_role()
{
    $_SESSION[gsp_get_applied_role_key()] = GSPMgr::getCurrentUrlAppliedRole();
}

//this function will be called while parsing the assets
//mangager will be able to fech this value
function gsp_set_matched_roles()
{
    $_SESSION[gsp_get_matched_role_key()] = GSPMgr::getCurrentUrlMatchedRoles();
}


function gsp_manager_render()
{
    if (!current_user_can('manage_options')) {
        die('Only Admins');
    }


    gsp_start_session();

    require_once(__DIR__ . '/helpers.php');
    require_once(__DIR__ . '/gsp_mgr.php');

    $url = gsp_get_active_url();

    if (!$url) {
        die('URL not sent');
    }

    $url = gsp_remove_param_from_url($url, "gspmgr-mode");
    $url = gsp_remove_param_from_url($url, "gspmgr-role");
    $url = gsp_cleanUrl($url);

    $testUrl = $url;
    $testUrl = gsp_add_url_param($testUrl, 'gspmgr-ignore', '1');
    $testUrl = gsp_add_url_param($testUrl, 'gspmgr-test-mode', '1');

    $assets = GSPMgr::parseUrlAssets($testUrl);


    if (!$assets['success']) {
        die('Error: ' . $assets['msg']);
    }

    $currentRole = gsp_get_cur_role();

    $appliedRole = gsp_get_applied_role();
    $matchedRoles = gsp_get_matched_roles();

    $roleExists = ($currentRole == GSPMgr::ROLE_ALL) ? true : GSPMgr::roleExists($currentRole);

    if ($roleExists || $currentRole == GSPMgr::ROLE_ALL) {
        GSPMgr::reset();
        GSPMgr::load($currentRole);
    } else {
        GSPMgr::reset();
        GSPMgr::load($appliedRole ? $appliedRole->getKey() :  GSPMgr::ROLE_ALL);
    }

    $roles = GSPMgr::getRoleList();

    $currentRoleObj = GSPMgr::findRole($currentRole);

?>
    <script>
        window.currentRoleObj = <?php echo json_encode($currentRoleObj->toArray()); ?>;
    </script>
    <div id="gspmgr-tool" class="gspmgr-bounce-in-top">
        <div class="gspmgr-add-role-dialog-container">

            <div class="gspmgr-add-role-dialog">
                <h2 id="gspmgr-add-new-role-title">Create New Role</h2>
                <input type="hidden" id="gsp-add-role-key">
                <label>
                    New Role Name
                    <input type="text" id="gsp-add-role-name">
                </label>
                <label>
                    Role URLS (use * for widecard, one per line)
                    <textarea rows="10" name="role-urls" id="gsp-add-role-urls"></textarea>
                </label>

                <div>
                    <button class="gspmgr-btn gspmgr-btn-primary" id="gspmgr-save-new-role">Save</button>
                    <button class="gspmgr-btn gspmgr-btn-secondary gsp-pull-right" id="gspmgr-new-role-cancel">Cancel</button>
                </div>
            </div>
        </div>

        <div class="gspmgr-tool-content">
            <div class="gspmgr-tool-body">
                <div class="gspmgr-header-url">
                    Current URL: <input readonly class="gspmgr-cururl-value" type="text" value="<?php echo $url; ?>">
                </div>
                <div>
                    Matched rules: <span id="gspmgr-matched-roles">
                        <?php foreach ($matchedRoles as $srole) : ?>
                            <a href="#" data-role="<?php echo $srole->getKey() ?>"><?php echo $srole->getName() ?></a>
                        <?php endforeach; ?>
                    </span>

                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                    Applied Rule: <span id="gspmgr-matched-roles">
                        <?php if ($appliedRole) : ?>
                            <a href="#" data-role="<?php echo $appliedRole->getKey() ?>"><?php echo $appliedRole->getName() ?></a>
                        <?php endif; ?>
                    </span>
                </div>

                <div class="gspmgr-header-rules">
                    <div>
                        Select Role:
                        <select id="gspmgr-roles">
                            <?php
                            foreach ($roles as $gsprole) :
                            ?>
                                <option <?php if ($gsprole->getKey() == $currentRole) echo 'selected' ?> value="<?php echo $gsprole->getKey() ?>">
                                    <?php echo ucwords(esc_attr($gsprole->getName())); ?>
                                    <?php if (GSPMgr::roleExists($gsprole->getKey())) echo '*' ?>
                                </option>
                            <?php endforeach; ?>
                        </select>


                        <button class="gspmgr-btn gspmgr-btn-secondary" id="gspmgr-add-role">Add</button>

                        <?php if (GSPMgr::roleExists($currentRole)) : ?>
                            <button class="gspmgr-btn gspmgr-btn-secondary" id="gspmgr-delete-role">Delete</button>
                        <?php endif; ?>

                        <?php if (!$currentRoleObj->isSystemRole()) : ?>
                            <button class="gspmgr-btn gspmgr-btn-secondary" id="gspmgr-edit-role">Edit</button>
                        <?php endif; ?>


                        <button class="gspmgr-btn gspmgr-btn-secondary" id="gspmgr-stop">Stop Manager</button>
                    </div>
                </div>

                <div class="gspmgr-asset-list-container">
                    <div class="gspmgr-header-buttons">
                        <a href="#" class="gspmgr-header-button gspmgr-active" data-target="#gspmgr-scripts-list">Scripts (<?php echo count($assets['scripts']); ?>)</a>
                        <a href="#" class="gspmgr-header-button" data-target="#gspmgr-styles-list">Styles (<?php echo count($assets['styles']); ?>)</a>
                    </div>
                    <div class="gspmgr-asset-list gspmgr-visible" id="gspmgr-scripts-list">
                        <?php
                        $lastgroup = null;
                        foreach ($assets['scripts'] as $script) :
                            $scriptid = md5($script);
                            $group = gsp_get_wordpress_url_group($script);
                            $groupkey = $group['type'] . '/' . $group['name'];
                            $asset = GSPMgr::getAsset($script, GSPMgr::TYPE_JS, $currentRole);
                        ?>
                            <?php if ($lastgroup != $group['name']) : ?>
                                <div class="gspmgr-asset-element-group" data-group="<?php echo $groupkey; ?>">
                                    <input data-group="<?php echo $groupkey ?>" type="checkbox" class="gspmgr-asset-element-group-toggle">
                                    <?php echo $group['name'] ?> <small><?php echo ucfirst($group['type']) ?></small>
                                </div>
                            <?php endif; ?>
                            <div data-group="<?php echo $groupkey ?>" data-id="<?php echo $scriptid ?>" title="<?php echo $script ?>" class="gspmgr-asset-element">
                                <label class="gspmgr-label-url">
                                    <input class="gspmgr-toggle-asset gspmgr-toggle-script" value="<?php echo esc_attr($script) ?>" <?php if (!$asset || !$asset->disabled) echo 'checked'; ?> type="checkbox" id="<?php echo esc_attr($scriptid); ?>">
                                    <span class="gspmgr-label-url-value">
                                        <?php
                                        echo gsp_short_link($script); ?>
                                    </span>
                                </label>

                                <div class="gspmgr-asset-element-actions">
                                    <select data-script-pos="<?php echo esc_attr($script) ?>" id="<?php echo esc_attr($scriptid) ?>-position">
                                        <option value="">Default Position</option>
                                        <option value="header" <?php if ($asset && $asset->position == 'header') echo 'selected'; ?>>Header</option>
                                        <option value="footer" <?php if ($asset && $asset->position == 'footer') echo 'selected'; ?>>Footer</option>
                                    </select>
                                    <label><input class="gspmgr-asset-element-action-async" data-script-async="<?php echo esc_attr($script); ?>" id="<?php echo esc_attr($scriptid); ?>-async" type="checkbox" value="1" <?php if ($asset && $asset->async) echo 'checked' ?>>Async</label>
                                    <label><input class="gspmgr-asset-element-action-defer" data-script-defer="<?php echo esc_attr($script); ?>" id="<?php echo esc_attr($scriptid); ?>-defer" type="checkbox" value="1" <?php if ($asset && $asset->defer) echo 'checked' ?>>Defer</label>
                                </div>
                            </div>
                        <?php
                            $lastgroup = $group['name'];
                        endforeach;
                        ?>
                    </div>

                    <div class="gspmgr-asset-list" id="gspmgr-styles-list">
                        <?php
                        $lastgroup = null;
                        foreach ($assets['styles'] as $style) :
                            $styleid = md5($style);
                            $groupkey = $group['type'] . '/' . $group['name'];
                            $group = gsp_get_wordpress_url_group($style);
                            $asset = GSPMgr::getAsset($style,  GSPMgr::TYPE_CSS, $currentRole);

                        ?>
                            <?php if ($lastgroup != $group['name']) : ?>
                                <div class="gspmgr-asset-element-group" data-group="<?php echo $groupkey; ?>">
                                    <input data-group="<?php echo $groupkey ?>" type="checkbox" class="gspmgr-asset-element-group-toggle">
                                    <?php echo $group['name'] ?> <small><?php echo ucfirst($group['type']) ?></small>
                                </div>
                            <?php endif; ?>
                            <div data-group="<?php echo $groupkey; ?>" data-id="<?php echo $styleid ?>" title="<?php echo esc_attr($style); ?>" class="gspmgr-asset-element">
                                <label class="gspmgr-label-url">
                                    <input class="gspmgr-toggle-asset gspmgr-toggle-style" value="<?php echo esc_attr($style); ?>" <?php if (!$asset || !$asset->disabled) echo 'checked'; ?> type="checkbox" id="<?php echo $styleid ?>">
                                    <span class="gspmgr-label-url-value"><?php echo gsp_short_link(esc_attr($style)); ?></span>
                                </label>
                                <div class="gspmgr-asset-element-actions">
                                    <select data-style-pos="<?php echo esc_attr($style) ?>" id="<?php echo esc_attr($styleid) ?>-position">
                                        <option value="">Default Position</option>
                                        <option value="header" <?php if ($asset && $asset->position == 'header') echo 'selected' ?>>Header</option>
                                        <option value="footer" <?php if ($asset && $asset->position == 'footer') echo 'selected' ?>>Footer</option>
                                    </select>
                                    <label><input class="gspmgr-asset-element-action-async" data-style-async="<?php echo esc_attr($style) ?>" id="<?php echo esc_attr($styleid) ?>-async" type="checkbox" value="1" <?php if ($asset && $asset->async) echo 'checked' ?>>Async</label>
                                    <label><input class="gspmgr-asset-element-action-defer" data-style-defer="<?php echo esc_attr($style) ?>" id="<?php echo esc_attr($styleid) ?>-defer" type="checkbox" value="1" <?php if ($asset && $asset->defer) echo 'checked' ?>>Defer</label>
                                </div>
                            </div>
                        <?php
                            $lastgroup  = $group['name'];
                        endforeach;
                        ?>
                    </div>

                    <div class="gspmgr-tool-footer">
                        <button class="gspmgr-btn gspmgr-btn-primary" id="gspmgr-save" type="button">Save & refresh</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php
}
